#if !defined(__RM9K_ETH_H__)
#define __RM9K_ETH_H__

#define RM9K_GE_NAME			"rm9k_ge"

/* Resource names */
#define RM9K_GE_RESOURCE_MAC      	"rm9k_ge_mac"
#define RM9K_GE_RESOURCE_MSTAT      	"rm9k_ge_mstat"
#define RM9K_GE_RESOURCE_PKTPROC	"rm9k_ge_pktproc"
#define RM9K_GE_RESOURCE_XDMA		"rm9k_ge_xdma"
#define RM9K_GE_RESOURCE_FIFO_RX  	"rm9k_ge_fifo_rx"
#define RM9K_GE_RESOURCE_FIFO_TX  	"rm9k_ge_fifo_tx"
#define RM9K_GE_RESOURCE_FIFOMEM_RX  	"rm9k_ge_fifo_memory_rx"
#define RM9K_GE_RESOURCE_FIFOMEM_TX  	"rm9k_ge_fifo_memory_tx"
#define RM9K_GE_RESOURCE_PHY      	"rm9k_ge_phy"
#define RM9K_GE_RESOURCE_DMADESC_RX  	"rm9k_ge_dmadesc_rx"
#define RM9K_GE_RESOURCE_DMADESC_TX  	"rm9k_ge_dmadesc_tx"
#define RM9K_GE_RESOURCE_IRQ_MAIN	"rm9k_ge_irq_main"
#define RM9K_GE_RESOURCE_IRQ_PHY	"rm9k_ge_irq_phy"
#define RM9K_GE_RESOURCE_GPI_SLICE	"rm9k_ge_gpi_slice"
#define RM9K_GE_RESOURCE_MDIO_CHANNEL	"rm9k_ge_mdio_channel"

#endif /* !defined(__RM9K_ETH_H__) */
