/*
 *  arch/arm/mach-aaec2000/include/mach/vmalloc.h
 *
 *  Copyright (c) 2005 Nicolas Bellido Y Ortega
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */

#ifndef __ASM_ARCH_VMALLOC_H
#define __ASM_ARCH_VMALLOC_H

#define VMALLOC_END		(PAGE_OFFSET + 0x10000000)

#endif /* __ASM_ARCH_VMALLOC_H */
